<?php

declare(strict_types=1);

namespace Drupal\Tests\crowd\Functional;

use Drupal\crowd\Crowd\CrowdConnectorInterface;
use Drupal\user\UserInterface;

/**
 * Defines a class for testing profile form changes.
 *
 * @group crowd
 * @covers \Drupal\crowd\Hooks\ProfileForm
 */
final class ProfileFormTest extends CrowdTestBase {

  /**
   * Test editing email address.
   */
  public function testEditingEmail(): void {
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->crowdConnector->register($email, $email, $pass, $given_name, $surname, $display_name);
    $this->crowdConnector->verifyUser($email);
    $account = \Drupal::service('externalauth.externalauth')->register($email, CrowdConnectorInterface::PROVIDER, [
      'name' => $email,
      'mail' => $email,
      // Set a random password that doesn't allow local login.
      'pass' => $this->randomMachineName(50),
      'status' => 1,
      'given_name' => $given_name,
      'surname' => $surname,
      'display_name' => $display_name,
    ], []);
    assert($account instanceof UserInterface);
    $account->passRaw = $pass;
    $this->drupalLogin($account);
    $this->drupalGet($account->toUrl('edit-form'));

    // Attempt to update email without adding password.
    $new_email = $this->randomEmail();
    $this->submitForm(['mail' => $new_email], 'Save');
    $assert = $this->assertSession();
    $assert->pageTextContains('Your current password is missing or incorrect');
    $this->submitForm([
      'mail' => $new_email,
      'current_pass' => 'yeh this is not the password',
    ], 'Save');
    $assert->pageTextContains('Your current password is missing or incorrect');
    $this->assertNull($this->crowdConnector->getUser($new_email));

    $this->submitForm([
      'mail' => $new_email,
      'current_pass' => $pass,
    ], 'Save');
    $assert->pageTextNotContains('Your current password is missing or incorrect');
    $assert->pageTextContains('The changes have been saved.');
    $this->assertNotNull($this->crowdConnector->getUser($new_email));
  }

  /**
   * Test editing password.
   */
  public function testEditingPassword(): void {
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->crowdConnector->register($email, $email, $pass, $given_name, $surname, $display_name);
    $this->crowdConnector->verifyUser($email);
    $account = \Drupal::service('externalauth.externalauth')->register($email, CrowdConnectorInterface::PROVIDER, [
      'name' => $email,
      'mail' => $email,
      // Set a random password that doesn't allow local login.
      'pass' => $this->randomMachineName(50),
      'status' => 1,
      'given_name' => $given_name,
      'surname' => $surname,
      'display_name' => $display_name,
    ], []);
    assert($account instanceof UserInterface);
    $account->passRaw = $pass;
    $this->drupalLogin($account);
    $this->drupalGet($account->toUrl('edit-form'));

    // Attempt to update password without current password.
    $new_pass = $this->randomMachineName(15);
    $this->submitForm([
      'pass[pass1]' => $new_pass,
      'pass[pass2]' => $new_pass,
    ], 'Save');
    $assert = $this->assertSession();
    $assert->pageTextContains('Your current password is missing or incorrect');
    $this->submitForm([
      'pass[pass1]' => $new_pass,
      'pass[pass2]' => $new_pass,
      'current_pass' => 'yeh this is not the password',
    ], 'Save');
    $assert->pageTextContains('Your current password is missing or incorrect');

    $this->submitForm([
      'pass[pass1]' => $new_pass,
      'pass[pass2]' => $new_pass,
      'current_pass' => $pass,
    ], 'Save');
    $assert->pageTextNotContains('Your current password is missing or incorrect');
    $assert->pageTextContains('The changes have been saved.');
    $this->drupalLogout();
    $account->passRaw = $new_pass;
    $this->drupalLogin($account);
  }

  /**
   * Tests changing email to/from a restricted domain.
   *
   * @dataProvider providerRestricted
   */
  public function testRestrictedDomain(bool $from_restricted): void {
    $restricted = sprintf('%s@%s', $this->randomMachineName(), $this->restrictedDomain);
    $not_restricted = $this->randomEmail();
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $from_restricted ? $restricted : $not_restricted;
    $pass = $this->randomMachineName(15);
    $this->crowdConnector->register($email, $email, $pass, $given_name, $surname, $display_name);
    $this->crowdConnector->verifyUser($email);
    $account = \Drupal::service('externalauth.externalauth')->register($email, CrowdConnectorInterface::PROVIDER, [
      'name' => $email,
      'mail' => $email,
      // Set a random password that doesn't allow local login.
      'pass' => $this->randomMachineName(50),
      'status' => 1,
      'given_name' => $given_name,
      'surname' => $surname,
      'display_name' => $display_name,
    ], []);
    assert($account instanceof UserInterface);
    $account->passRaw = $pass;
    $this->drupalLogin($account);
    $this->drupalGet($account->toUrl('edit-form'));

    // Attempt to update email without adding password.
    $new_email = $from_restricted ? $not_restricted : $restricted;
    $this->submitForm([
      'mail' => $new_email,
      'current_pass' => $pass,
    ], 'Save');
    $assert = $this->assertSession();

    $assert->pageTextContains(sprintf('Email address cannot be changed for users from the %s domain.', $this->restrictedDomain));
    $this->assertNull($this->crowdConnector->getUser($new_email));
  }

  /**
   * Tests editing profile fields.
   */
  public function testEditingProfileFields(): void {
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->crowdConnector->register($email, $email, $pass, $given_name, $surname, $display_name);
    $this->crowdConnector->verifyUser($email);
    $account = \Drupal::service('externalauth.externalauth')->register($email, CrowdConnectorInterface::PROVIDER, [
      'name' => $email,
      'mail' => $email,
      // Set a random password that doesn't allow local login.
      'pass' => $this->randomMachineName(50),
      'status' => 1,
      'given_name' => $given_name,
      'surname' => $surname,
      'display_name' => $display_name,
    ], []);
    assert($account instanceof UserInterface);
    $account->passRaw = $pass;
    $this->drupalLogin($account);
    $this->drupalGet($account->toUrl('edit-form'));

    $new_name = $this->randomMachineName();
    $this->submitForm(['given_name[0][value]' => $new_name], 'Save');
    $assert = $this->assertSession();
    $assert->pageTextContains('The changes have been saved.');
    $user = $this->crowdConnector->getUser($email);
    $this->assertEquals($new_name, $user->getData()['first-name']);
    $account = \Drupal::entityTypeManager()->getStorage('user')->loadUnchanged($account->id());
    $this->assertEquals($new_name, $account->given_name->value);
  }

  /**
   * Data provider for ::testRestrictedDomain().
   *
   * @return array[]
   *   Test cases.
   */
  public function providerRestricted(): array {
    return [
      'from restricted' => [
        TRUE,
      ],
      'to restricted' => [
        FALSE,
      ],
    ];
  }

}
