<?php

declare(strict_types=1);

namespace Drupal\Tests\crowd\Functional;

use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;
use Drupal\user\UserInterface;

/**
 * Defines a class for testing registration.
 *
 * @group crowd
 * @covers \Drupal\crowd\Hooks\Login
 * @covers \Drupal\crowd\Hooks\Register
 * @covers \Drupal\crowd\Hooks\AccountSync
 */
final class RegisterTest extends CrowdTestBase {

  /**
   * Tests registration as a user.
   */
  public function testRegisterAsUser(): void {
    $this->drupalGet('/user/register');
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $this->submitForm([
      'mail' => $email,
      'given_name[0][value]' => $given_name,
      'surname[0][value]' => $surname,
      'display_name[0][value]' => $display_name,
    ], 'Create new account');
    $user = user_load_by_mail($email);
    $this->assertInstanceOf(UserInterface::class, $user);
    $this->assertTrue($user->isActive());
    $this->assertEquals($given_name, $user->get('given_name')->value);
    $this->assertEquals($surname, $user->get('surname')->value);
    $this->assertEquals($display_name, $user->get('display_name')->value);
    $new_display_name = $this->randomMachineName();
    $user->set('display_name', $new_display_name)->save();
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertFalse($remote_user->isActive());
    $mails = $this->getMails(['to' => $email]);
    $this->assertCount(1, $mails);
    $mail = reset($mails);
    $reset_url = $this->getResetUrl($mail);
    $this->assertNotNull($reset_url);
    $this->drupalGet($reset_url);
    $this->submitForm([], 'Log in');
    $this->assertSession()->pageTextContains('You have just used your one-time login link. It is no longer necessary to use this link to log in. Please set your password.');
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertTrue($remote_user->isActive());
    // Reload user, display name should have synced from crowd.
    $user = \Drupal::entityTypeManager()->getStorage('user')->loadUnchanged($user->id());
    $this->assertEquals($display_name, $user->get('display_name')->value);
  }

  /**
   * Tests registration as an admin with notify.
   */
  public function testRegisterAsAdminWithNotify(): void {
    $this->drupalLogin($this->drupalCreateUser([
      'administer users',
      'access administration pages',
    ]));
    $this->drupalGet('/admin/people/create');
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->submitForm([
      'mail' => $email,
      'name' => $email,
      'notify' => TRUE,
      'given_name[0][value]' => $given_name,
      'surname[0][value]' => $surname,
      'display_name[0][value]' => $display_name,
      'pass[pass1]' => $pass,
      'pass[pass2]' => $pass,
    ], 'Create new account');
    $user = user_load_by_mail($email);
    $this->assertInstanceOf(UserInterface::class, $user);
    $this->assertTrue($user->isActive());
    $this->assertEquals($given_name, $user->get('given_name')->value);
    $this->assertEquals($surname, $user->get('surname')->value);
    $this->assertEquals($display_name, $user->get('display_name')->value);
    $new_display_name = $this->randomMachineName();
    $user->set('display_name', $new_display_name)->save();
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertFalse($remote_user->isActive());
    $mails = $this->getMails(['to' => $email]);
    $this->assertCount(1, $mails);
    $mail = reset($mails);
    $reset_url = $this->getResetUrl($mail);
    $this->assertNotNull($reset_url);
    $this->drupalLogout();

    $this->drupalGet($reset_url);
    $this->submitForm([], 'Log in');
    $this->assertSession()->pageTextContains('You have just used your one-time login link. It is no longer necessary to use this link to log in. Please set your password.');
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertTrue($remote_user->isActive());
    // Reload user, display name should have synced from crowd.
    $user = \Drupal::entityTypeManager()->getStorage('user')->loadUnchanged($user->id());
    $this->assertEquals($display_name, $user->get('display_name')->value);
  }

  /**
   * Tests registration as an admin without notify.
   */
  public function testRegisterAsAdminWithoutNotify(): void {
    $this->drupalLogin($this->drupalCreateUser([
      'administer users',
      'access administration pages',
    ]));
    $this->drupalGet('/admin/people/create');
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->submitForm([
      'mail' => $email,
      'name' => $email,
      'notify' => FALSE,
      'given_name[0][value]' => $given_name,
      'surname[0][value]' => $surname,
      'display_name[0][value]' => $display_name,
      'pass[pass1]' => $pass,
      'pass[pass2]' => $pass,
    ], 'Create new account');
    $user = user_load_by_mail($email);
    $this->assertInstanceOf(UserInterface::class, $user);
    $this->assertTrue($user->isActive());
    $this->assertEquals($given_name, $user->get('given_name')->value);
    $this->assertEquals($surname, $user->get('surname')->value);
    $this->assertEquals($display_name, $user->get('display_name')->value);
    $new_display_name = $this->randomMachineName();
    $user->set('display_name', $new_display_name)->save();
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertFalse($remote_user->isActive());
    $mails = $this->getMails(['to' => $email]);
    $this->assertCount(0, $mails);
    $this->drupalLogout();

    $this->drupalGet('user/password');
    $this->submitForm(['name' => $email], 'Submit');

    $mails = $this->getMails(['to' => $email]);
    $this->assertCount(1, $mails);
    $mail = reset($mails);
    $reset_url = $this->getResetUrl($mail);
    $this->assertNotNull($reset_url);

    $this->drupalGet($reset_url);
    $this->submitForm([], 'Log in');
    $this->assertSession()->pageTextContains('You have just used your one-time login link. It is no longer necessary to use this link to log in. Please set your password.');
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNotNull($remote_user);
    $this->assertTrue($remote_user->isActive());
    // Reload user, display name should have synced from crowd.
    $user = \Drupal::entityTypeManager()->getStorage('user')->loadUnchanged($user->id());
    $this->assertEquals($display_name, $user->get('display_name')->value);
  }

  /**
   * Tests registration a local accounts an admin.
   */
  public function testRegisterLocalAccountAsAdmin(): void {
    $this->drupalLogin($this->drupalCreateUser([
      'administer users',
      'access administration pages',
      'administer permissions',
    ]));
    $role = Role::create([
      'id' => mb_strtolower($this->randomMachineName()),
      'label' => $this->randomMachineName(),
    ]);
    assert($role instanceof RoleInterface);
    $role->grantPermission('login to local drupal account');
    $role->save();
    $this->drupalGet('/admin/people/create');
    $given_name = $this->randomMachineName();
    $surname = $this->randomMachineName();
    $display_name = $this->randomMachineName();
    $email = $this->randomEmail();
    $pass = $this->randomMachineName(15);
    $this->submitForm([
      'mail' => $email,
      'name' => $email,
      sprintf('roles[%s]', $role->id()) => TRUE,
      'notify' => FALSE,
      'local_account' => TRUE,
      'given_name[0][value]' => $given_name,
      'surname[0][value]' => $surname,
      'display_name[0][value]' => $display_name,
      'pass[pass1]' => $pass,
      'pass[pass2]' => $pass,
    ], 'Create new account');
    $user = user_load_by_mail($email);
    $this->assertInstanceOf(UserInterface::class, $user);
    $this->assertTrue($user->isActive());
    $this->assertEquals($given_name, $user->get('given_name')->value);
    $this->assertEquals($surname, $user->get('surname')->value);
    $this->assertEquals($display_name, $user->get('display_name')->value);
    $remote_user = $this->crowdConnector->getUser($email);
    $this->assertNull($remote_user);
    $this->drupalLogout();

    $this->drupalGet('user/login');
    $user->passRaw = $pass;
    $this->drupalLogin($user);
  }

}
